"""
    Patches OFS.Image.File so that an "Image Convert"
    tab is available for all objects based on it.
"""
# $Id: ImageConvertPatch.py,v 1.3 2001/10/21 00:11:07 adrian Exp $
#
# $Log: ImageConvertPatch.py,v $
# Revision 1.3  2001/10/21 00:11:07  adrian
# Fixed bug where "content_type" was inherited if it was not found.
#
# Revision 1.2  2001/10/21 00:04:17  adrian
# no message
#
#
__version__ = '$Revision: 1.3 $'[11:-2]

import OFS.Image
import Globals
import whrandom
import Acquisition

def filterImageConvertTab(obj):
    if hasattr(Acquisition.aq_base(obj), 'content_type'):
        if (obj.content_type[:6]!='image/'):
            return 0
    return 1

convertOptions = ['Image']

def manage_performImageConvert(self, convert='Image', inplace='0', keepold='0', REQUEST=None):
    """ Attempts to perform an ImageConvert operation. """
    parent = self.aq_parent
    oldName = "%s" % self.id()
    destName = "%s" % self.id()
    data = self.data
    while hasattr(parent, destName):
        destName = "%s_%d" % (self.id(), whrandom.uniform(1000, 10000))
    if inplace!='0':
        tmpName = oldName
        oldName = destName
        destName = tmpName
        parent.manage_renameObject(destName, oldName)
    if convert == 'Image':
        OFS.Image.manage_addImage(parent, id=destName, title=self.title, file=self.data)
    else:
        raise "Conversion Error", "What did you want to convert TO?"
    if keepold=='0':
        parent.manage_delObjects([oldName])
    if REQUEST:
        if inplace=='1':
            if keepold=='0':
                message="Converted to %s called '%s' (Original deleted)" % (convert, destName)
                item=getattr(parent, destName)
            else:
                message="Converted to %s called '%s' (Original renamed to %s)" % (convert, destName, oldName)
                item=getattr(parent, oldName)
        else:
            message="Converted to %s called '%s' (Original unchanged)" % (convert, destName)
            item=getattr(parent, oldName)
        return item.manage_main(item, REQUEST, manage_tabs_message=message)
    return None

targets = (
        OFS.Image.File,
)
manage_options = (
    {'label':'Image Convert', 'action':'manage_performImageConvertForm',
     'filter':filterImageConvertTab, 'help':('PatchKit', 'ImageConvert.stx'),
    },
)
methods = {
    'manage_performImageConvertForm':
        Globals.DTMLFile('www/imageConvert', globals()),
    'manage_performImageConvert': manage_performImageConvert,
    'imageConvertOptions': convertOptions,
}
