"""
    PackageUpdate Patch

This patch adds a new Control_Panel object that checks for
new versions of installed products.

(This patch was inspired by Andy McKay's ZPM product)
"""
# $Id: PackageUpdate.py,v 1.5 2001/11/29 10:45:52 adrian Exp $
#
# $Log: PackageUpdate.py,v $
# Revision 1.5  2001/11/29 10:45:52  adrian
# Fixed bug where empty XML or a failed XML download caused an
# error message - This now acts as though the server has no content.
#
# Revision 1.4  2001/10/28 22:46:31  adrian
# Merged product version reading code.
# Modified product version reading to support non-numeric versions.
# Added indicator for "No update available" in Package Update.
# Added link to Product Package if local is newer.
# Added list of products not to list.
# Set version to 4.2.0
#
# Revision 1.3  2001/10/26 12:51:20  adrian
# * Added caching using XMLKit's XMLProxy object. (Initial cache-timer set to 1440 minutes == 24 hours)
# * Added multiple views of the update list.
# * Added a re-cache tab/command.
# * Improved layout.
# * Product icon indicates product is installed.
# * This version does not auto-install the products, but allows you to download
# them to your workstation for review first (A good idea IMHO).
#
# Revision 1.2  2001/10/22 06:38:09  adrian
# no message
#
#
__version__ = '$Revision: 1.5 $'[11:-2]

import OFS.SimpleItem
import App.ApplicationManager
import Globals
import string
import os
import Acquisition
import Products
import ProductPackage

try:
# This is currently NON-OPTIONAL
    import Products.XMLKit.XML.XMLParser

# Use XMLKit to parse data from products_xml on zope.org
# Have data cached etc.

## XML Structure
## <products>
##  <product>
##   <id>ProductID</id>
##   <name>Product Title</name>
##   <creator>Creator Name</creator>
##   <modified>Zope format date</modified>
##   <url>Software Product URL</url>
##   <latest-file>Latest File URL</latest-file>
##   <version>Latest Version</version>
##  </product>
## </products>

    class PackageUpdate(App.ApplicationManager.Fake, OFS.SimpleItem.SimpleItem, Acquisition.Implicit):
        """ Search for updated packages """
        id          = "PackageUpdates"
        title       = "Package Updates"
        meta_type   = "Package Updates"
        icon        = "p_/ProductFolder_icon"
        xmlparser   = Products.XMLKit.XMLProxy.XMLProxy('PU_XML', 'http://www.zope.org/Members/haqa/products_xml')
        xmlparser.reloadtimer = 1440 # Cache for 24 hours - Speeds up display A LOT!!!
        xmlparser.autoreload = 1
        products    = {}
        name        = title
        manage_updated = Globals.DTMLFile('www/PackageUpdate', globals(), group=1)
        manage_new  = Globals.DTMLFile('www/PackageUpdate', globals(), group=2)
        manage_installed = Globals.DTMLFile('www/PackageUpdate', globals(), group=3)
        manage_all = Globals.DTMLFile('www/PackageUpdate', globals(), group=4)

        manage_options = (
            (
                {'label':'Updated Products', 'action':'manage_updated',
                 'help':('PatchKit','ProductUpdates.stx')},
                {'label':'New Products', 'action':'manage_new',
                 'help':('PatchKit','ProductUpdates.stx')},
                {'label':'Installed Products', 'action':'manage_installed',
                 'help':('PatchKit','ProductUpdates.stx')},
                {'label':'All Products', 'action':'manage_all',
                 'help':('PatchKit','ProductUpdates.stx')},
                {'label':'Reload Cached Products', 'action':'manage_recache', }
            )
        )
        __ac_permissions__ = (
            ('View',
                ('getProductDetails', )
            ),
            ('View management screens',
                ('manage_recache', 'manage_updated', 'manage_new',
                 'manage_installed', 'manage_all', )
            ),
        )

        def manage_recache(self, REQUEST=None):
            """ Force a reload of the XML data """
            self.xmlparser._reload(2)
            if REQUEST:
                REQUEST.RESPONSE.redirect('manage_workspace')
            

        def getProductDetails(self, group=0):
            """ Return the product list """
            info=[]
# Get products_xml
            if self.xmlparser._auto_reload() or not self.products:
# convert to mapping keyed on "id"
                prods = {}
# DEBUG: Raise if there are duplicate product names...
                DOM = self.xmlparser._internal_getDOMref()
                root = DOM.getNextChildByName('products')
                if root:
                    contents = root.getChildrenByName()
                    for item in contents:
                        if item.getNextChildByName('id').getCDATA() not in excludedProducts:
                            prods[item.getNextChildByName('id').getCDATA()] = {
                                'id': item.getNextChildByName('id').getCDATA(),
                                'name': item.getNextChildByName('name').getCDATA(),
                                'creator': item.getNextChildByName('creator').getCDATA(),
                                'modified': item.getNextChildByName('modified').getCDATA(),
                                'url': item.getNextChildByName('url').getCDATA(),
                                'latest-file': item.getNextChildByName('latest-file').getCDATA(),
                                'version': item.getNextChildByName('version').getCDATA(),
                            }
                self.products = prods
            prods = {}
            for productsFolder in Products.__path__:
                products = os.listdir(productsFolder)
                products.sort()
                for product in products:
                    if product not in excludedProducts:
                        productFolder = os.path.join(productsFolder, product)
                        if os.path.isdir(productFolder):
                            version = ''
                            prods[product] = {
                                'product_name': str(product),
                                'local_location': str(productFolder),
                                'local_version': str(ProductPackage.productVersion(self, product, [productsFolder])),
                                'remote_version': '',
                                'remote_file': '',
                                'remote_home': '',
                                'remote_creator': '',
                                'remote_modified': '',
                            }
            for item in self.products.keys():
                if not prods.has_key(item):
                    prods[item] = {
                        'product_name': str(item),
                        'local_location': '',
                        'local_version': '',
                    }
                prods[item]['remote_version'] = self.products[item]['version']
                prods[item]['remote_file'] = self.products[item]['latest-file']
                prods[item]['remote_home'] = self.products[item]['url']
                prods[item]['remote_creator'] = self.products[item]['creator']
                prods[item]['remote_modified'] = self.products[item]['modified']
            prodKeys = prods.keys()
            prodKeys.sort()
            for item in prodKeys:
                if (
                        (group==1 and prods[item]['local_location'] and prods[item]['remote_file'] and
                         prods[item]['remote_version']>prods[item]['local_version']) or
                        (group==2 and not prods[item]['local_location'] and prods[item]['remote_file']) or
                        (group==3 and prods[item]['local_location']) or
                        (group==4)
                    ):
                    info.append(prods[item])
            return info

    Globals.default__class_init__(PackageUpdate)

# Log: Added fix to remove _objects from Control_Panel instance
# Question: Does hasattr find attributes of class aswell as instance

    def performPatch(app):
        """ Patch the ApplicationManager """
        list = []
        cp = app.Control_Panel
        try:
            del cp._objects
        except:
            pass
        am = App.ApplicationManager.ApplicationManager
        am.Updates = PackageUpdate()
        for item in am._objects:
            if item['id'] != 'Updates':
                list.append(item)
        list.append({'id': 'Updates', 'meta_type': PackageUpdate.meta_type, })
        am._objects = tuple(list)
except:
    pass

excludedProducts = (
    'CVS',              # Don't want to see my CVS folders
    'ExternalMethod',   # Std Product
    'MIMETools',
    'MailHost',
    'OFSP',
    'PluginIndexes',
    'PythonScripts',
    'SiteAccess',
    'StandardCacheManagers',
    'ZCatalog',
    'ZGadflyDA',
    'ZSQLMethods',
    'ZopeTutorial',
)
