"""
    Patches products to provide a way to package FS products TTW

Thanks go to:
    Pavel Zaichenko <zaichep@mail.com>
        Patch to solve INSTANCE_HOME problem.
"""
# $Id: ProductPackage.py,v 1.3 2001/10/28 22:46:31 adrian Exp $
#
# $Log: ProductPackage.py,v $
# Revision 1.3  2001/10/28 22:46:31  adrian
# Merged product version reading code.
# Modified product version reading to support non-numeric versions.
# Added indicator for "No update available" in Package Update.
# Added link to Product Package if local is newer.
# Added list of products not to list.
# Set version to 4.2.0
#
# Revision 1.2  2001/10/21 10:13:34  adrian
# Add source code control files to the list of "excluded files".
# "CVS" directories, and Visual Source Safe ".scc" files will no longer
# get added to package tar-balls.
#
#
__version__ = '$Revision: 1.3 $'[11:-2]

import os
import string
import App.Product
import Products
import Globals
import App.tar

class MyTGZ(App.tar.tgzarchive):
    """ Simple derived class that just changes the owner """
    def add(self, name, data):
        self._f.write(str(App.tar.TarEntry(name, data, uname="The", gname="Haqa")))

def manage_makePackage(self, REQUEST, RESPONSE, submit=None):
    """ Build a package tarball """
    ap, items = fileLists(self)
    tar = MyTGZ(self.productFilename(0))
    entries = []
    for item in items:
        item = os.path.normpath(item)
        file = open(os.path.join(ap, item))
        unixName = string.join(string.split(item, os.sep), '/')
        tar.add(unixName, file.read())
        file.close()
    tar.finish()
    data = tar.__str__()
    RESPONSE.setHeader('Content-Type', 'application/x-gzip-compressed; name="%s"' % self.productFilename(1))
    RESPONSE.setHeader('Content-Disposition', 'inline; filename="%s"' % self.productFilename(1))
    RESPONSE.setHeader('Content-Length', '%d' % len(data))
    RESPONSE.write(data)
    return ''

def _getFiles(folder, lead):
    """ Walk this folder """
    files = []
    for file in os.listdir(folder):
        if os.path.isdir(os.path.join(folder,file)):
            if file != 'CVS':
                files.extend(_getFiles(os.path.join(folder, file),
                    os.path.join(lead, file)))
        else:
            files.append(os.path.join(lead, file))
    return files

def fileLists(self, prodId=None):
    """ Return a tuple containing (folder, included_files) """
    if not prodId:
        prodId = self.getId()
    escape = "let_me_out"
    incl = []
    try:
        for folder in Products.__path__:
            productFolder = os.path.join(folder, prodId)
            if os.path.isdir(productFolder):
                for file in _getFiles(productFolder, prodId):
                    if os.path.splitext(file)[1] not in excludeExtensions:
                        incl.append(file)
                raise escape, (folder, incl)
    except escape, (f, i):
        incl.sort()
        return (f, i)
    return (None, [])

def productVersionFile(self, prodId=None, paths=Products.__path__):
    """ """
    if not prodId:
        prodId = self.getId()
    version = None
    folder, dummy = fileLists(self, prodId)
    p = os.path.join(folder, prodId, 'version.txt')
    if not os.path.exists(p):
        p = os.path.join(folder, prodId, 'VERSION.txt')
    if not os.path.exists(p):
        p = os.path.join(folder, prodId, 'VERSION.TXT')
    if os.path.exists(p):
        try:
            file = open(p)
            text = file.read()
            file.close()
            version = text
        except:
            pass
    return version

def _cleanVersion(version, prodId=None):
    """ """
    if not prodId:
        prodId = self.getId()
    version = string.replace(version, '-', ' ')
    version = string.replace(version, '.', ' ')
    version = string.replace(version, "%s " % prodId, "")
    version = string.replace(version, "Version ", "")
    vList = string.split(version, ' ')
    version = vList[0]
    for i in range(1, len(vList)):
        try:
            val = string.atoi(vList[i-1]) + string.atoi(vList[i])
            version = version + ".%s" % vList[i]
        except:
            version = version + "-%s" % vList[i]
    return string.strip(version)

def productVersion(self, prodId=None, paths=Products.__path__):
    """ """
    if not prodId:
        prodId = self.getId()
    version = productVersionFile(self, prodId, paths)
    if not version:
        version = '1.0.0'
    version = _cleanVersion(version, prodId)
    return version

def productFilename(self, includeExtension=1):
    """ Return the filename for this tarball """
    name = "%s-%s" % (self.getId(), self.productVersion(self.getId()))
    if includeExtension:
        name = name + ".tgz"
    return name

def initializeProduct(productp, name, home, app):
    """ Replacement Product Initializer """
    target = App.Product.pkInitializeProduct(productp, name, home, app)
    newOptions = []
    if hasattr(target, 'manage_options'):
        for option in target.manage_options:
            if (not option.has_key('label') or
                option['label'] != 'Package'):
                newOptions.append(option)
        newOptions.append(
            {'label': 'Package', 'action': 'manage_makePackageForm',
             'help':('PatchKit', 'Package.stx'),
            }
        )
        target.manage_options = tuple(newOptions)
    return target

if not hasattr(App.Product, 'pkInitializeProduct'):
    App.Product.pkInitializeProduct = App.Product.initializeProduct
App.Product.initializeProduct = initializeProduct

excludeExtensions = (
    '.pyc',
    '.pyo',
    '.bak',
    '.py~',
    '.orig',
    '.old',
    '.swp',
    '.tmp',
    '.scc',
)

targets = (
    App.Product.Product,
)
methods = {
    'manage_makePackageForm':   Globals.HTMLFile('www/Package', globals()),
    'manage_makePackage':       manage_makePackage,
    'excludeExtensions':        excludeExtensions,
    'fileLists':                fileLists,
    'productVersion':           productVersion,
    'productVersionFile':       productVersionFile,
    'productFilename':          productFilename,
}
